package com.supermap.imobile.findpath;

import com.supermap.analyst.networkanalyst.TransportationAnalyst;
import com.supermap.analyst.networkanalyst.TransportationAnalystParameter;
import com.supermap.analyst.networkanalyst.TransportationAnalystResult;
import com.supermap.analyst.networkanalyst.TransportationAnalystSetting;
import com.supermap.analyst.networkanalyst.WeightFieldInfo;
import com.supermap.analyst.networkanalyst.WeightFieldInfos;
import com.supermap.data.Color;
import com.supermap.data.CoordSysTransMethod;
import com.supermap.data.CoordSysTransParameter;
import com.supermap.data.CoordSysTranslator;
import com.supermap.data.DatasetVector;
import com.supermap.data.Datasource;
import com.supermap.data.DatasourceConnectionInfo;
import com.supermap.data.EngineType;
import com.supermap.data.Environment;
import com.supermap.data.GeoLineM;
import com.supermap.data.GeoStyle;
import com.supermap.data.Point;
import com.supermap.data.Point2D;
import com.supermap.data.Point2Ds;
import com.supermap.data.PrjCoordSys;
import com.supermap.data.PrjCoordSysType;
import com.supermap.data.Workspace;
import com.supermap.mapping.CallOut;
import com.supermap.mapping.CalloutAlignment;
import com.supermap.mapping.Layer;
import com.supermap.mapping.LayerSettingVector;
import com.supermap.mapping.MapControl;
import com.supermap.mapping.MapView;
import com.supermap.mapping.TrackingLayer;
import android.app.Activity;
import android.app.ProgressDialog;
import android.os.Bundle;
import android.view.GestureDetector;
import android.view.MotionEvent;
import android.view.View;
import android.view.GestureDetector.SimpleOnGestureListener;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.Toast;
/**
 * <p>
 * Title: Shortest Path Analyst
 * </p>
 * 
 * <p>
 * Description:
 * ============================================================================>
 * ------------------------------Statement----------------------------
 * This is for SuperMap iMobile for Android Sample Code 
 * Copyright: SuperMap Software Co., Ltd.
 * ----------------------------------------------------------------
 * ---------------------SuperMap iMobile for Android Sample Code------------------------
 * 
 * 1. Inroduction: How to perform shortest path analyst
 * 2. Data: /sdcard/SampleData/City/Changchun.udb
 * 3. Members: 
 *		TransportationAnalystSetting.setNetworkDataset()	Method
 *		TransportationAnalystSetting.setEdgeIDField()		Method
 *		TransportationAnalystSetting.setNodeIDField()		Method
 *		TransportationAnalystSetting.setEdgeNameField()		Method
 *		TransportationAnalystSetting.setWeightFieldInfos()	Method
 *		TransportationAnalystSetting.setFNodeIDField()		Method
 *		TransportationAnalystSetting.setTNodeIDField()		Method
 *		TransportationAnalyst.setAnalystSetting()			Method
 *		TransportationAnalyst.load()						Method
 *		TransportationAnalyst.findPath()					Method
 *		TransportationAnalystParameter.setPoints()			Method
 *		TransportationAnalystParameter.setNodesReturn()		Method
 *		TransportationAnalystParameter.setEdgesReturn()		Method
 *		TransportationAnalystParameter.setPathGuidesReturn()Method
 *		TransportationAnalystParameter.setRoutesReturn()	Method

 * 4. Steps:
 *   (1) Click to set the start button. Long press the map to select start point
 *   (2) Add target button. Long press the map to select the target points (you can select multiple target points)
 *   (3) Click the path analysis button. The result is displayed on the map
 *   (4) Clear the result
 *   (5) Next analysis
 * ------------------------------------------------------------------------------
 * ============================================================================>
 * </p> 
 * 
 * <p>
 * Company: SuperMap Software Co., Ltd.
 * </p>
 * 
 */
public class MainFrame extends Activity  implements OnClickListener {
    
	private Workspace mWorkspace = null;	
	private MapView mMapView = null;
	private MapControl mMapControl = null;	
	
	private Button btnRoute = null;
	private Button btnSetting = null;
	private Button btnClean = null;
	private Button btnDsetting = null;
	
	//Status changes
	private boolean bEndPointEnable = false;
	private boolean bAnalystEnable = false;
	private boolean bLongPressEnable = false;
	//Can not modify start and end points after path analysis
	private boolean bSettingEnable = true;
	
	private DatasetVector m_datasetLine;
	private Layer m_layerLine;
	private TrackingLayer m_trackingLayer;	
	private Point2Ds m_Points = null;
	private static String m_datasetName = "RoadNet";

	private static String m_nodeID = "SmNodeID";
	private static String m_edgeID = "SmEdgeID";	
	private TransportationAnalyst m_analyst;
	private TransportationAnalystResult m_result;
	private final String sdcard = android.os.Environment.getExternalStorageDirectory().getAbsolutePath().toString();

	//Receive long press event
	private GestureDetector.SimpleOnGestureListener mGestrueListener = new SimpleOnGestureListener(){
		public void onLongPress(MotionEvent e) {
			if(!bLongPressEnable){
				return;
			}
			int x = (int) e.getX();
			int y = (int) e.getY();
			Point2D pt = mMapControl.getMap().pixelToMap(new Point(x, y));
			CallOut callout = new CallOut(MainFrame.this);
			callout.setStyle(CalloutAlignment.LEFT_BOTTOM);
			callout.setCustomize(true);
			callout.setLocation(pt.getX(),pt.getY());
			//When the projection is not the longitude and latitude coordinate system, you should transfer start and end points first.
			if(mMapControl.getMap().getPrjCoordSys().getType() != PrjCoordSysType.PCS_EARTH_LONGITUDE_LATITUDE){
				Point2Ds points = new Point2Ds();
				points.add(pt);
				PrjCoordSys desPrjCoorSys = new PrjCoordSys();
				desPrjCoorSys.setType(PrjCoordSysType.PCS_EARTH_LONGITUDE_LATITUDE);
				CoordSysTranslator.convert(points, mMapControl.getMap().getPrjCoordSys(), desPrjCoorSys, new CoordSysTransParameter(), CoordSysTransMethod.MTH_GEOCENTRIC_TRANSLATION);
				pt = points.getItem(0);
			}
			ImageView image = new ImageView(MainFrame.this);
			// Add the first point
			if(!bEndPointEnable){
				image.setBackgroundResource(R.drawable.startpoint);
				callout.setContentView(image);
				mMapView.addCallout(callout);
				m_Points.add(pt);//Add the target point
				bEndPointEnable = true;
				bLongPressEnable = false;
				btnSetting.setText("Add target point");
				btnSetting.invalidate();
				return;
			}
			image.setBackgroundResource(R.drawable.despoint);
			callout.setContentView(image);
			mMapView.addCallout(callout);
			m_Points.add(pt);//Add the target point
			bAnalystEnable = true;
			btnSetting.setText("Add target point");
			bLongPressEnable = false;
		};
	};
	
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        // Set path
        Environment.setLicensePath(sdcard+"/SuperMap/license/");
        //Set paths
        Environment.initialization(this);
        
        setContentView(R.layout.main);
        initUI();
        initialize();
    }
    
    @Override
    protected void onStart() {
    	// TODO Auto-generated method stub
    	final ProgressDialog dialog = new ProgressDialog(this);
		dialog.setCancelable(false);
		dialog.setCanceledOnTouchOutside(false);
		dialog.setMessage("Loading traffic network analysis object...");
		dialog.show();
		new Thread(new Runnable() {
			
			@Override
			public void run() {
				// Load network model
				load();
				dialog.dismiss();
			}
		}).start();
		
    	super.onStart();
    }
    
    private void initUI(){
    	mMapView = (MapView)findViewById(R.id.mapview);
    	mMapControl = mMapView.getMapControl();
    	btnRoute = (Button) findViewById(R.id.analyst);
    	btnRoute.setOnClickListener(this);
    	btnSetting = (Button) findViewById(R.id.setting);
    	btnSetting.setOnClickListener(this);
    	btnClean = (Button)findViewById(R.id.btn_bufferClean);
    	btnClean.setOnClickListener(new OnClickListener() {
			
			@Override
			public void onClick(View v) {
				// TODO Auto-generated method stub
				mMapControl.getMap().getTrackingLayer().clear();
				mMapControl.getMap().refresh();
			}
		});
    	btnDsetting = (Button)findViewById(R.id.Dsetting);
    	btnDsetting.setOnClickListener(new OnClickListener() {
			
			@Override
			public void onClick(View v) {
				// TODO Auto-generated method stub
				bSettingEnable = true;
				bEndPointEnable = false;
				bAnalystEnable = false;
				bLongPressEnable = true;
				mMapView.removeAllCallOut();
				m_Points.clear();
				btnSetting.setText("Set start point");
				btnRoute.setText("Path analysis");
			}
		});
    	
    	m_Points = new Point2Ds();
    }
    
    
    /**
	 * Open the network dataset and initialize the corresponding variables
	 */
	private void initialize()
	{
    	mWorkspace = new Workspace();
    	
		//Open the datasource to get point and line datasets
		DatasourceConnectionInfo info = new DatasourceConnectionInfo();
		String dataServer = sdcard+"/SampleData/City/Changchun.udb";
	
		info.setEngineType(EngineType.UDB);
		info.setServer(dataServer);
		Datasource datasource = mWorkspace.getDatasources().open(info);
    	
    	mMapControl.getMap().setWorkspace(mWorkspace);
   
		m_datasetLine = (DatasetVector)datasource.getDatasets().get(m_datasetName);
		m_trackingLayer = mMapControl.getMap().getTrackingLayer();

		

		//Load the line dataset and set styles
		m_layerLine = mMapControl.getMap().getLayers().add(m_datasetLine,
				true);
		m_layerLine.setSelectable(false);
		LayerSettingVector lineSetting = (LayerSettingVector)m_layerLine
				.getAdditionalSetting();
		GeoStyle lineStyle = new GeoStyle();
		lineStyle.setLineColor(new Color(0, 0, 255));
		lineStyle.setLineWidth(0.1);
		lineSetting.setStyle(lineStyle);
		
    	mMapControl.getMap().viewEntire();
		mMapControl.getMap().refresh();

		//Set the gesture entrust
    	mMapControl.setGestureDetector(new GestureDetector(mGestrueListener));
	}
	
	/**
	 * Load the environnment setting object
	 */
	public void load()
	{
		// Set basic environment of network analysis
		TransportationAnalystSetting setting = new TransportationAnalystSetting();
		setting.setNetworkDataset(m_datasetLine);
		setting.setEdgeIDField(m_edgeID);
		setting.setNodeIDField(m_nodeID);
		setting.setEdgeNameField("roadName");
	
		setting.setTolerance(89);

		WeightFieldInfos weightFieldInfos = new WeightFieldInfos();
		WeightFieldInfo weightFieldInfo = new WeightFieldInfo();
		weightFieldInfo.setFTWeightField("smLength");
		weightFieldInfo.setTFWeightField("smLength");
		weightFieldInfo.setName("length");
		weightFieldInfos.add(weightFieldInfo);
		setting.setWeightFieldInfos(weightFieldInfos);
		setting.setFNodeIDField("SmFNode");
		setting.setTNodeIDField("SmTNode");

		//Construct the traffic network analysis object. Load environment setting object
		m_analyst = new TransportationAnalyst();
		m_analyst.setAnalystSetting(setting);
		m_analyst.load();

	}

	/**
	 * Shortest path analysis
	 */
	public boolean analyst()
	{
		TransportationAnalystParameter parameter = new TransportationAnalystParameter();
		
		parameter.setWeightName("length");

		//Set the return object of shortest path analysis
		parameter.setPoints(m_Points);
		parameter.setNodesReturn(true);
		parameter.setEdgesReturn(true);
		parameter.setPathGuidesReturn(true);
		parameter.setRoutesReturn(true);

		try{
		//Analyze and display results
			m_result = m_analyst.findPath(parameter, false);
		}
		catch(Exception e){
			m_result = null;
		}
		if (m_result == null)
		{
			toastInfo("Failed to analyze");
			return false;
		}
		showResult();
		return true;
	}

	private void toastInfo(final String msg) {
		runOnUiThread(new Runnable() {
			
			@Override
			public void run() {
				// TODO Auto-generated method stub
				Toast.makeText(MainFrame.this, msg, Toast.LENGTH_SHORT).show();
			}
		});
		
	}
	
	/**
	 * Display result
	 */
	public void showResult()
	{

		//Delete the original results
		int count = m_trackingLayer.getCount();
		for (int i = 0; i < count; i++)
		{
			int index = m_trackingLayer.indexOf("result");
			if (index != -1)
				m_trackingLayer.remove(index);
		}

		GeoLineM[] routes = m_result.getRoutes();
		
		if (routes == null) {
			return;
		}
		
		for (int i = 0; i < routes.length; i++)
		{
			GeoLineM geoLineM = routes[i];
			GeoStyle style = new GeoStyle();
			style.setLineColor(new Color(255, 80, 0));
			style.setLineWidth(1);
			geoLineM.setStyle(style);
			m_trackingLayer.add(geoLineM, "result");
		}
		
		mMapControl.getMap().refresh();
	}
	
	@Override
	public void onClick(View btn) {
		switch (btn.getId()) {
		case R.id.analyst:
			if(!bAnalystEnable){
				Toast.makeText(this, "Please set start and end points first!", Toast.LENGTH_SHORT).show();
				return;
			}
			
			final ProgressDialog dialog = new ProgressDialog(this);
			dialog.setCancelable(false);
			dialog.setCanceledOnTouchOutside(false);
			dialog.setMessage("Path analyzing...");
			dialog.show();
			new Thread(new Runnable() {
				
				@Override
				public void run() {
					//Shortest path analysis
					analyst();
					dialog.dismiss();
				}
			}).start();
			
			break;
		case R.id.setting:
			if(!bSettingEnable){
				Toast.makeText(this, "Can not modify start and end points!", Toast.LENGTH_SHORT).show();
				bLongPressEnable = true;
				return;
			}
			if(bEndPointEnable){
				Toast.makeText(this, "Long press to add end point!", Toast.LENGTH_SHORT).show();
				bLongPressEnable = true;
				return;
			}
			Toast.makeText(this, "Long press to add start point!", Toast.LENGTH_SHORT).show();
			bLongPressEnable = true;
			break;
		default: 
			break;
		}
	}
	
}